==> react-translated-2.1.0.tgz: package/lib/renderer.js <==
// @flow

import type { TranslatedType } from './constants'
import basicParser from 'basic-parser'
import * as React from 'react'

//////////////////
// MATCH RENDER //
//////////////////

type MatchRenderPropsType = {
  children: mixed[],
  data: {
    onPressLink: () => void,
    templateData?: Object,
    renderMap?: { [string]: (?Object) => React.Node },
  },
  startMatches: string[],
  endMatches: string[],
}

const getRendererForBoldText = ({ Component }) => ({
  match: '\\*',
  render({ children }: MatchRenderPropsType) {
    return <Component.BoldText>{children}</Component.BoldText>
  },
})

const getRendererForLinkText = ({ Component }) => ({
  match: { start: '\\[', end: '\\]\\((.*?)\\)' },
  render({ children, data, endMatches }: MatchRenderPropsType) {
    const url = endMatches[0] || ''
    const onPress = data.onPressLink
    return (
      <Component.Link onPress={onPress} url={url}>
        {children}
      </Component.Link>
    )
  },
})

const getRendererForCustomComponents = () => ({
  match: { start: '<', end: '>' },
  render({ children, data }: MatchRenderPropsType) {
    const componentName = typeof children[0] === 'string' ? children[0] : ''
    const render = data.renderMap && data.renderMap[`render${componentName}`]
    if (!render) {
      throw new Error(`Render method "render${componentName}" not passed`)
    }
    return render(data.templateData)
  },
})

//////////////
// TEMPLATE //
//////////////

const getTemplate = ({ data, language, text, translation }) => {
  // Grab the template from the translation data
  // and default to the text if none is found.
  let template = translation[text] ? translation[text][language] : undefined
  template = template || text

  // If it's a function, it is a dynamic template, so invoke it with the data
  // and return the final template
  return typeof template === 'string' ? template : template(data)
}

const variableRenderer = {
  match: { start: '\\{', end: '\\}' },
  render({ data, children }: { data: Object, children: string }) {
    return data[children] || ''
  },
}
const renderVariables = basicParser({ renderers: [variableRenderer] })

////////////
// CREATE //
////////////

export type RenderPropsType = {
  text: string,
  data?: Object,
  render?: ({ translatedText: string }) => React.Node,
  renderMap?: { [string]: () => React.Node },
  onPressLink?: () => mixed,
  translated: TranslatedType,
}

type CreateType = ({
  Component: { [string]: React.ComponentType<*> },
}) => RenderPropsType => React.Node

export const create: CreateType = ({ Component }) => {
  const renderers = [
    getRendererForBoldText({ Component }),
    getRendererForLinkText({ Component }),
    getRendererForCustomComponents(),
  ]
  const renderWithRenderers = basicParser({ renderers })
  return ({
    text,
    data,
    render,
    renderMap,
    translated,
    onPressLink,
  }: RenderPropsType) => {
    const translation = translated.getTranslation()
    const language = translated.getLanguage()
    if (!translation || !language) {
      throw new Error(
        'No translation data provided. Make sure your <Provider> is set up correctly.',
      )
    }

    let template = getTemplate({ data, language, text, translation })
    template = renderVariables({ data, text: template }).join('')

    if (render) {
      return render({ translatedText: template })
    }

    const chunks = renderWithRenderers({
      data: { templateData: data, onPressLink, renderMap },
      text: template,
    }).map(
      (chunk, index) =>
        typeof chunk === 'string'
          ? chunk
          : React.cloneElement(chunk, { key: index }),
    )

    const isDebugging = translated.getIsDebugging()
    const isMissing = !translation[text] || !translation[text][language]
    if (!isDebugging || !isMissing) {
      return chunks
    }
    const style = { backgroundColor: 'red' }
    return <Component.Text style={style}>{chunks}</Component.Text>
  }
}

==> react-translated-2.2.1.tgz: package/lib/renderer.js <==
// @flow

import basicParser from 'basic-parser'
import * as React from 'react'

///////////
// MATCH //
///////////

type MatchRenderPropsType = {
  children: mixed[],
  data: {
    onPressLink: () => void,
    templateData?: Object,
    renderMap?: { [string]: (?Object) => React.Node },
  },
  startMatches: string[],
  endMatches: string[],
}

type GetRendererType<PropsType = void> = PropsType => {
  match: string | { start: string, end: string },
  render: MatchRenderPropsType => React.Node,
}

export const getForBoldText: GetRendererType<{ Component: Object }> = ({
  Component,
}) => ({
  match: '\\*',
  render({ children }: MatchRenderPropsType) {
    return <Component.BoldText>{children}</Component.BoldText>
  },
})

export const getForLinkText: GetRendererType<{ Component: Object }> = ({
  Component,
}) => ({
  match: { start: '\\[', end: '\\]\\((.*?)\\)' },
  render({ children, data, endMatches }: MatchRenderPropsType) {
    const url = endMatches[0] || ''
    const onPress = data.onPressLink
    return (
      <Component.Link onPress={onPress} url={url}>
        {children}
      </Component.Link>
    )
  },
})

export const getForCustomComponents: GetRendererType<> = () => ({
  match: { start: '<', end: '>' },
  render({ children, data }: MatchRenderPropsType) {
    const componentName = typeof children[0] === 'string' ? children[0] : ''
    const render = data.renderMap && data.renderMap[`render${componentName}`]
    if (!render) {
      throw new Error(`Render method "render${componentName}" not passed`)
    }
    return render(data.templateData)
  },
})

///////////////
// VARIABLES //
///////////////

const variablesRenderer = {
  match: { start: '\\{', end: '\\}' },
  render({ data, children }: { data: Object, children: string }) {
    return data[children] || ''
  },
}

const renderVariables = basicParser({
  renderers: [variablesRenderer],
})

type InterpolateDataType = ({ data?: Object, text: string }) => string

export const interpolateData: InterpolateDataType = ({ data, text }) =>
  renderVariables({ data, text }).join('')
